@@ -5,34 +5,33 @@
  *  Copyright © 2004 Crispin Flowerday
  *  Copyright © 2004 Adam Hooper
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2, or (at your option)
- *  any later version.
+ *  This file is part of Epiphany.
  *
- *  This program is distributed in the hope that it will be useful,
+ *  Epiphany is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ *
+ *  Epiphany is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
- *
+ *  along with Epiphany.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
 #include "ephy-embed-utils.h"
 
 #include "ephy-about-handler.h"
-#include "ephy-embed-private.h"
 #include "ephy-settings.h"
 #include "ephy-string.h"
 
-#include <string.h>
+#include <JavaScriptCore/JavaScript.h>
 #include <glib/gi18n.h>
 #include <libsoup/soup.h>
-#include <JavaScriptCore/JavaScript.h>
+#include <string.h>
 
 static GRegex *non_search_regex;
 static GRegex *domain_regex;
@@ -40,7 +39,6 @@ static GRegex *domain_regex;
 char *
 ephy_embed_utils_link_message_parse (const char *message)
 {
-  
   char *status_message;
   char **splitted_message;
   int i = 1;
@@ -48,7 +46,7 @@ ephy_embed_utils_link_message_parse (const char *message)
   GString *tmp;
 
   status_message = ephy_string_blank_chr (g_strdup (message));
-  
+
   if (!status_message || !g_str_has_prefix (status_message, "mailto:"))
     return status_message;
 
@@ -57,14 +55,14 @@ ephy_embed_utils_link_message_parse (const char *message)
    */
   p = strchr (status_message, '?');
   if (p != NULL) *p = '\0';
-    
+
   /* Then we also want to check if there is more than an email address
    * in the mailto: list.
    */
   splitted_message = g_strsplit_set (status_message, ";", -1);
   tmp = g_string_new (g_strdup_printf (_("Send an email message to “%s”"),
                                        (splitted_message[0] + 7)));
-    
+
   while (splitted_message [i] != NULL) {
     g_string_append_printf (tmp, ", “%s”", splitted_message[i]);
     i++;
@@ -139,7 +137,7 @@ gboolean
 ephy_embed_utils_address_is_existing_absolute_filename (const char *address)
 {
   return g_path_is_absolute (address) &&
-    g_file_test (address, G_FILE_TEST_EXISTS);
+         g_file_test (address, G_FILE_TEST_EXISTS);
 }
 
 static gboolean
@@ -169,6 +167,34 @@ is_public_domain (const char *address)
   return retval;
 }
 
+static gboolean
+is_bang_search (const char *address)
+{
+  EphyEmbedShell *shell;
+  EphySearchEngineManager *search_engine_manager;
+  char **bangs;
+  GString *buffer;
+
+  shell = ephy_embed_shell_get_default ();
+  search_engine_manager = ephy_embed_shell_get_search_engine_manager (shell);
+  bangs = ephy_search_engine_manager_get_bangs (search_engine_manager);
+
+  for (uint i = 0; bangs[i] != NULL; i++) {
+    buffer = g_string_new (bangs[i]);
+    g_string_append (buffer, " ");
+
+    if (strstr (address, buffer->str) == address) {
+      g_string_free (buffer, TRUE);
+      g_free (bangs);
+      return TRUE;
+    }
+    g_string_free (buffer, TRUE);
+  }
+  g_free (bangs);
+
+  return FALSE;
+}
+
 gboolean
 ephy_embed_utils_address_is_valid (const char *address)
 {
@@ -181,22 +207,33 @@ ephy_embed_utils_address_is_valid (const char *address)
   scheme = g_uri_parse_scheme (address);
 
   retval = scheme ||
-    ephy_embed_utils_address_is_existing_absolute_filename (address) ||
-    g_regex_match (get_non_search_regex (), address, 0, NULL) ||
-    is_public_domain (address);
+           ephy_embed_utils_address_is_existing_absolute_filename (address) ||
+           g_regex_match (get_non_search_regex (), address, 0, NULL) ||
+           is_public_domain (address) ||
+           is_bang_search (address);
 
   g_free (scheme);
 
   return retval;
 }
 
-char*
+char *
 ephy_embed_utils_normalize_address (const char *address)
 {
   char *effective_address = NULL;
 
   g_return_val_if_fail (address, NULL);
 
+  if (is_bang_search (address)) {
+    EphyEmbedShell *shell;
+    EphySearchEngineManager *search_engine_manager;
+
+    shell = ephy_embed_shell_get_default ();
+    search_engine_manager = ephy_embed_shell_get_search_engine_manager (shell);
+    return ephy_search_engine_manager_parse_bang_search (search_engine_manager,
+                                                         address);
+  }
+
   if (ephy_embed_utils_address_is_existing_absolute_filename (address))
     return g_strconcat ("file://", address, NULL);
 
@@ -231,24 +268,26 @@ ephy_embed_utils_normalize_address (const char *address)
 char *
 ephy_embed_utils_autosearch_address (const char *search_key)
 {
-  char *query_param, *url_search;
+  char *query_param;
+  const char *address_search;
   char *effective_address;
+  EphyEmbedShell *shell;
+  EphySearchEngineManager *search_engine_manager;
 
-  url_search = g_settings_get_string (EPHY_SETTINGS_MAIN,
-                                      EPHY_PREFS_KEYWORD_SEARCH_URL);
-  if (url_search == NULL || url_search[0] == '\0') {
-    g_free (url_search);
-    url_search = g_strdup (_("https://duckduckgo.com/?q=%s&amp;t=epiphany"));
-  }
+  shell = ephy_embed_shell_get_default ();
+  search_engine_manager = ephy_embed_shell_get_search_engine_manager (shell);
+  address_search = ephy_search_engine_manager_get_default_search_address (search_engine_manager);
 
   query_param = soup_form_encode ("q", search_key, NULL);
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wformat-nonliteral"
+  /* Format string under control of user input... but gsettings is trusted input. */
   /* + 2 here is getting rid of 'q=' */
-  effective_address = g_strdup_printf (url_search, query_param + 2);
+  effective_address = g_strdup_printf (address_search, query_param + 2);
+#pragma GCC diagnostic pop
   g_free (query_param);
-  g_free (url_search);
 
   return effective_address;
-
 }
 
 char *
@@ -263,19 +302,16 @@ ephy_embed_utils_normalize_or_autosearch_address (const char *address)
 gboolean
 ephy_embed_utils_url_is_empty (const char *location)
 {
-  gboolean is_empty = FALSE;
-
-  if (location == NULL || 
-      location[0] == '\0' || 
-      strcmp (location, "about:blank") == 0)
-    is_empty = TRUE;
-  
-  return is_empty;
+  return (location == NULL ||
+          location[0] == '\0' ||
+          strcmp (location, "about:blank") == 0 ||
+          strcmp (location, "ephy-about:overview") == 0 ||
+          strcmp (location, "ephy-about:incognito") == 0);
 }
 
 /* This is the list of addresses that should never be shown in the
  * window's location entry. */
-static const char * do_not_show_address[] = {
+static const char *do_not_show_address[] = {
   "about:blank",
   "ephy-about:incognito",
   "ephy-about:overview",
@@ -303,9 +339,9 @@ ephy_embed_utils_get_title_from_address (const char *address)
   if (g_str_has_prefix (address, "file://"))
     return g_strdup (address + 7);
 
-  if (!strcmp (address, EPHY_ABOUT_SCHEME":overview") ||
+  if (!strcmp (address, EPHY_ABOUT_SCHEME ":overview") ||
       !strcmp (address, "about:overview"))
-    return g_strdup (_("Most Visited"));
+    return g_strdup (_(OVERVIEW_PAGE_TITLE));
 
   return ephy_string_get_host_name (address);
 }
@@ -322,26 +358,12 @@ ephy_embed_utils_urls_have_same_origin (const char *a_url,
     return retval;
 
   b_uri = soup_uri_new (b_url);
-  if (!b_uri) {
-    soup_uri_free (a_uri);
-    return retval;
-  }
-
-  if (a_uri->host && b_uri->host) {
-    retval = soup_uri_host_equal (a_uri, b_uri);
-    if (!retval) {
-      const char *a_domain;
-      const char *b_domain;
-
-      a_domain = soup_tld_get_base_domain (a_uri->host, NULL);
-      b_domain = soup_tld_get_base_domain (b_uri->host, NULL);
-
-      retval = a_domain && b_domain && strcmp (a_domain, b_domain) == 0;
-    }
+  if (b_uri) {
+    retval = a_uri->host && b_uri->host && soup_uri_host_equal (a_uri, b_uri);
+    soup_uri_free (b_uri);
   }
 
   soup_uri_free (a_uri);
-  soup_uri_free (b_uri);
 
   return retval;
 }
