@@ -1,20 +1,21 @@
 /* -*- Mode: C; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
-/* vim: set sw=2 ts=2 sts=2 et: */
 /*
  *  Copyright © 2013 Igalia S.L.
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2, or (at your option)
- *  any later version.
+ *  This file is part of Epiphany.
  *
- *  This program is distributed in the hope that it will be useful,
+ *  Epiphany is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ *
+ *  Epiphany is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ *  along with Epiphany.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -22,6 +23,34 @@
 
 #include <libsoup/soup.h>
 #include <stdio.h>
+#include <string.h>
+
+typedef struct {
+  WebKitDOMHTMLInputElement *element;
+  gint index;
+} InputElementData;
+
+static InputElementData *
+input_element_data_new (WebKitDOMHTMLInputElement *element,
+                        gint                       index)
+{
+  InputElementData *data;
+
+  data = g_slice_new (InputElementData);
+  data->element = g_object_ref (element);
+  data->index = index;
+
+  return data;
+}
+
+static void
+input_element_data_free (InputElementData *data)
+{
+  g_assert (data);
+
+  g_object_unref (data->element);
+  g_slice_free (InputElementData, data);
+}
 
 /**
  * ephy_web_dom_utils_has_modified_forms:
@@ -123,17 +152,17 @@ ephy_web_dom_utils_has_modified_forms (WebKitDOMDocument *document)
 char *
 ephy_web_dom_utils_get_application_title (WebKitDOMDocument *document)
 {
-  WebKitDOMNodeList *metas;
+  WebKitDOMHTMLCollection *metas;
   char *title = NULL;
   gulong length, i;
 
-  metas = webkit_dom_document_get_elements_by_tag_name (document, "meta");
-  length = webkit_dom_node_list_get_length (metas);
+  metas = webkit_dom_document_get_elements_by_tag_name_as_html_collection (document, "meta");
+  length = webkit_dom_html_collection_get_length (metas);
 
   for (i = 0; i < length && title == NULL; i++) {
     char *name;
     char *property;
-    WebKitDOMNode *node = webkit_dom_node_list_item (metas, i);
+    WebKitDOMNode *node = webkit_dom_html_collection_item (metas, i);
 
     /* og:site_name is read from the property attribute (standard), but is
      * commonly seen on the web in the name attribute. Both are supported. */
@@ -184,17 +213,17 @@ get_icon_from_html_icon (WebKitDOMDocument *document,
                          char             **uri_out)
 {
   gboolean ret;
-  WebKitDOMNodeList *links;
+  WebKitDOMHTMLCollection *links;
   gulong length, i;
   char *image = NULL;
   int largest_icon = 0;
 
-  links = webkit_dom_document_get_elements_by_tag_name (document, "link");
-  length = webkit_dom_node_list_get_length (links);
+  links = webkit_dom_document_get_elements_by_tag_name_as_html_collection (document, "link");
+  length = webkit_dom_html_collection_get_length (links);
 
   for (i = 0; i < length; i++) {
     char *rel;
-    WebKitDOMNode *node = webkit_dom_node_list_item (links, i);
+    WebKitDOMNode *node = webkit_dom_html_collection_item (links, i);
 
     rel = webkit_dom_html_link_element_get_rel (WEBKIT_DOM_HTML_LINK_ELEMENT (node));
     if (rel != NULL && (
@@ -256,19 +285,19 @@ get_icon_from_mstile (WebKitDOMDocument *document,
                       char             **color_out)
 {
   gboolean ret;
-  WebKitDOMNodeList *metas;
+  WebKitDOMHTMLCollection *metas;
   gulong length, i;
   char *image = NULL;
   char *color = NULL;
 
-  metas = webkit_dom_document_get_elements_by_tag_name (document, "meta");
-  length = webkit_dom_node_list_get_length (metas);
+  metas = webkit_dom_document_get_elements_by_tag_name_as_html_collection (document, "meta");
+  length = webkit_dom_html_collection_get_length (metas);
 
   /* FIXME: Ought to also search browserconfig.xml
    * See: http://stackoverflow.com/questions/24625305/msapplication-tileimage-favicon-backup
    */
   for (i = 0; i < length; i++) {
-    WebKitDOMNode *node = webkit_dom_node_list_item (metas, i);
+    WebKitDOMNode *node = webkit_dom_html_collection_item (metas, i);
     char *name;
 
     name = webkit_dom_html_meta_element_get_name (WEBKIT_DOM_HTML_META_ELEMENT (node));
@@ -300,15 +329,15 @@ get_icon_from_ogp (WebKitDOMDocument *document,
                    char             **uri_out)
 {
   gboolean ret;
-  WebKitDOMNodeList *metas;
+  WebKitDOMHTMLCollection *metas;
   gulong length, i;
   char *image = NULL;
 
-  metas = webkit_dom_document_get_elements_by_tag_name (document, "meta");
-  length = webkit_dom_node_list_get_length (metas);
+  metas = webkit_dom_document_get_elements_by_tag_name_as_html_collection (document, "meta");
+  length = webkit_dom_html_collection_get_length (metas);
 
   for (i = 0; i < length && image == NULL; i++) {
-    WebKitDOMNode *node = webkit_dom_node_list_item (metas, i);
+    WebKitDOMNode *node = webkit_dom_html_collection_item (metas, i);
     char *property;
     char *itemprop;
 
@@ -337,16 +366,16 @@ get_icon_from_touch_icon (WebKitDOMDocument *document,
                           char             **uri_out)
 {
   gboolean ret;
-  WebKitDOMNodeList *links;
+  WebKitDOMHTMLCollection *links;
   gulong length, i;
   char *image = NULL;
 
-  links = webkit_dom_document_get_elements_by_tag_name (document, "link");
-  length = webkit_dom_node_list_get_length (links);
+  links = webkit_dom_document_get_elements_by_tag_name_as_html_collection (document, "link");
+  length = webkit_dom_html_collection_get_length (links);
 
   for (i = 0; i < length && image == NULL; i++) {
     char *rel;
-    WebKitDOMNode *node = webkit_dom_node_list_item (links, i);
+    WebKitDOMNode *node = webkit_dom_html_collection_item (links, i);
 
     rel = webkit_dom_html_link_element_get_rel (WEBKIT_DOM_HTML_LINK_ELEMENT (node));
     /* TODO: support more than one possible icon. */
@@ -378,16 +407,16 @@ static void
 get_icon_from_favicon (WebKitDOMDocument *document,
                        char             **uri_out)
 {
-  WebKitDOMNodeList *links;
+  WebKitDOMHTMLCollection *links;
   gulong length, i;
   char *image = NULL;
 
-  links = webkit_dom_document_get_elements_by_tag_name (document, "link");
-  length = webkit_dom_node_list_get_length (links);
+  links = webkit_dom_document_get_elements_by_tag_name_as_html_collection (document, "link");
+  length = webkit_dom_html_collection_get_length (links);
 
   for (i = 0; i < length; i++) {
     char *rel;
-    WebKitDOMNode *node = webkit_dom_node_list_item (links, i);
+    WebKitDOMNode *node = webkit_dom_html_collection_item (links, i);
 
     rel = webkit_dom_html_link_element_get_rel (WEBKIT_DOM_HTML_LINK_ELEMENT (node));
     if (rel != NULL && (
@@ -435,6 +464,8 @@ ephy_web_dom_utils_get_best_icon (WebKitDOMDocument *document,
 
   /* FIXME: These functions could be improved considerably. See the first two answers at:
    * http://stackoverflow.com/questions/21991044/how-to-get-high-resolution-website-logo-favicon-for-a-given-url
+   *
+   * Also check out: https://www.slightfuture.com/webdev/gnome-web-app-icons
    */
   found_icon = get_icon_from_html_icon (document, &image);
   if (!found_icon)
@@ -454,17 +485,15 @@ ephy_web_dom_utils_get_best_icon (WebKitDOMDocument *document,
   g_free (image);
 }
 
-gboolean
-ephy_web_dom_utils_find_form_auth_elements (WebKitDOMHTMLFormElement *form,
-                                            WebKitDOMNode           **username,
-                                            WebKitDOMNode           **password)
+GPtrArray *
+ephy_web_dom_utils_find_password_fields (WebKitDOMHTMLFormElement *form,
+                                         AuthCacheMode             mode)
 {
+  GPtrArray *password_fields = NULL;
   WebKitDOMHTMLCollection *elements;
-  WebKitDOMNode *username_node = NULL;
-  WebKitDOMNode *password_node = NULL;
   guint i, n_elements;
-  gboolean found_auth_elements = FALSE;
-  gboolean found_auth_no_username_elements = FALSE;
+
+  password_fields = g_ptr_array_new_full (3, (GDestroyNotify)input_element_data_free);
 
   elements = webkit_dom_html_form_element_get_elements (form);
   n_elements = webkit_dom_html_collection_get_length (elements);
@@ -472,64 +501,179 @@ ephy_web_dom_utils_find_form_auth_elements (WebKitDOMHTMLFormElement *form,
   for (i = 0; i < n_elements; i++) {
     WebKitDOMNode *element;
     char *element_type;
-    char *element_name;
+    char *element_value;
+    InputElementData *data;
 
     element = webkit_dom_html_collection_item (elements, i);
     if (!WEBKIT_DOM_IS_HTML_INPUT_ELEMENT (element))
       continue;
 
-    g_object_get (element, "type", &element_type, "name", &element_name, NULL);
+    g_object_get (element, "type", &element_type, "value", &element_value, NULL);
+    if (g_strcmp0 (element_type, "password") != 0)
+      goto next;
 
-    if (g_strcmp0 (element_type, "text") == 0 ||
-        g_strcmp0 (element_type, "email") == 0 ||
-        g_strcmp0 (element_type, "tel") == 0) {
-      /* We found more than one inputs of type text; we won't be saving here. */
-      if (username_node) {
-        g_free (element_type);
-        found_auth_elements = FALSE;
-        break;
-      }
+    if ((element_value == NULL || strlen (element_value) == 0) &&
+        mode == AUTH_CACHE_SUBMIT)
+      goto next;
 
-      username_node = g_object_ref (element);
-      found_auth_elements = TRUE;
-    } else if (g_strcmp0 (element_type, "password") == 0) {
-      /* We found more than one inputs of type password; we won't be saving here. */
-      if (password_node) {
-        g_free (element_type);
-        found_auth_elements = FALSE;
-        break;
-      }
+    data = input_element_data_new (WEBKIT_DOM_HTML_INPUT_ELEMENT (element), i);
 
-      password_node = g_object_ref (element);
-      found_auth_elements = TRUE;
-
-      /* We found an input that usually doesn't require a separate login
-       * adminpw is used by mailman admin pages */
-      if (g_strcmp0 (element_name, "adminpw") == 0)
-        found_auth_no_username_elements = TRUE;
-    }
+    g_ptr_array_add (password_fields, data);
 
+next:
     g_free (element_type);
-    g_free (element_name);
+    g_free (element_value);
   }
 
   g_object_unref (elements);
 
-  if (found_auth_no_username_elements && password_node) {
-    g_clear_object (&username_node);
-    *username = NULL;
-    *password = password_node;
-
-    return TRUE;
+  /*
+   * We only want to process forms with 1-3 fields. A common
+   * case is to have a "change password" form with 3 fields:
+   * Old password, New password, Confirm new password.
+   * Forms with more than 3 password fields are unlikely,
+   * and we don't know how to process them, so reject them
+   */
+  if (password_fields->len == 0 || password_fields->len > 3) {
+    g_ptr_array_free (password_fields, TRUE);
+    return NULL;
   }
 
-  if (found_auth_elements && username_node && password_node) {
-    *username = username_node;
-    *password = password_node;
+  return password_fields;
+}
 
-    return TRUE;
+/*
+ * The heuristic is based on the one used in Firefox. See
+ * https://dxr.mozilla.org/mozilla-central/rev/892c8916ba32b7733e06bfbfdd4083ffae3ca028/toolkit/components/passwordmgr/LoginManagerContent.jsm#733
+ */
+gboolean
+ephy_web_dom_utils_find_form_auth_elements (WebKitDOMHTMLFormElement *form,
+                                            WebKitDOMNode           **username,
+                                            WebKitDOMNode           **password,
+                                            AuthCacheMode             mode)
+{
+  WebKitDOMHTMLCollection *elements;
+  WebKitDOMHTMLInputElement *username_node = NULL;
+  WebKitDOMHTMLInputElement *password_node = NULL;
+  gint i, password_node_index = 0;
+  GPtrArray *password_nodes = NULL;
+  gchar **passwords;
+  InputElementData *first_password_node_data = NULL;
+
+  password_nodes = ephy_web_dom_utils_find_password_fields (form, mode);
+  if (password_nodes == NULL)
+    return FALSE;
+
+  /*
+   * Start at the first found password field and search backwards.
+   * Assume the first eligible field to contain username
+   */
+  first_password_node_data = g_ptr_array_index (password_nodes, 0);
+  elements = webkit_dom_html_form_element_get_elements (form);
+  for (i = first_password_node_data->index; i >= 0; i--) {
+    WebKitDOMNode *element;
+    gchar *element_type;
+
+    element = webkit_dom_html_collection_item (elements, i);
+    if (!WEBKIT_DOM_IS_HTML_INPUT_ELEMENT (element))
+      continue;
+
+    g_object_get (element, "type", &element_type, NULL);
+    if (g_strcmp0 (element_type, "text") == 0 ||
+        g_strcmp0 (element_type, "email") == 0 ||
+        g_strcmp0 (element_type, "tel") == 0 ||
+        g_strcmp0 (element_type, "url") == 0 ||
+        g_strcmp0 (element_type, "number") == 0) {
+      username_node = g_object_ref (element);
+      g_free (element_type);
+      break;
+    }
+
+    g_free (element_type);
   }
 
+  /*
+   * Choose password field that contains the password that we want to store
+   * To do that, we compare the field values. We can only do this when user
+   * submits login data, because otherwise all the fields are empty. In that
+   * case just pick the first field
+   */
+  if (mode == AUTH_CACHE_AUTOFILL || password_nodes->len == 1) {
+    /* If mode == AUTH_CACHE_AUTOFILL, we expect all fields to be empty */
+    password_node_index = 0;
+  } else {
+    /* Get values of all password fields */
+    passwords = g_malloc0 (password_nodes->len * sizeof (gchar *));
+    for (i = password_nodes->len - 1; i >= 0; i--) {
+      WebKitDOMHTMLInputElement *element;
+      element = ((InputElementData *) g_ptr_array_index (password_nodes, i))->element;
+      passwords[i] = webkit_dom_html_input_element_get_value (element);
+    }
+
+    if (password_nodes->len == 2) {
+      /*
+       * If there are two password fields, assume that the form has either
+       * Password and Confirm password fields, or Old password and New password.
+       * That can be guessed by comparing values in the fields. If they are
+       * different, we assume that the second password is "new" and use it.
+       * If they match, then just take the first field.
+       */
+      if (g_strcmp0 (passwords[0], passwords[1]) == 0) {
+        /* Password / Confirm password */
+        password_node_index = 0;
+      } else {
+        /* Old password / New password */
+        password_node_index = 1;
+      }
+    } else if (password_nodes->len == 3) {
+      /*
+       * This is probably a complete Old password, New password, Confirm
+       * new password case. Here we assume that if two fields have the same
+       * value, then it's the new password and we should take it. A special
+       * case is when all 3 passwords are different. We don't know what to
+       * do in this case, so just reject the form,
+       */
+      if (g_strcmp0 (passwords[0], passwords[1]) == 0 &&
+          g_strcmp0 (passwords[1], passwords[2]) == 0) {
+        /* All values are same */
+        password_node_index = 0;
+      } else if (g_strcmp0 (passwords[0], passwords[1]) == 0) {
+        /* New password / Confirm new password / Old password */
+        password_node_index = 0;
+      } else if (g_strcmp0 (passwords[0], passwords[2]) == 0) {
+        /* New password / Old password / Confirm new password */
+        password_node_index = 0;
+      } else if (g_strcmp0 (passwords[1], passwords[2]) == 0) {
+        /* Old password / New password / Confirm new password */
+        password_node_index = 1;
+      } else {
+        /* All values are different. Reject the form */
+        password_node_index = -1;
+      }
+    }
+
+    for (i = 0; i < password_nodes->len; i++)
+      g_free (passwords[i]);
+    g_free (passwords);
+  }
+
+  if (password_node_index >= 0) {
+    InputElementData *password_data = g_ptr_array_index (password_nodes,
+                                                         password_node_index);
+    password_node = g_object_ref (password_data->element);
+  }
+
+  if (username_node)
+    *username = WEBKIT_DOM_NODE (username_node);
+  if (password_node)
+    *password = WEBKIT_DOM_NODE (password_node);
+
+  g_object_unref (elements);
+  g_ptr_array_free (password_nodes, TRUE);
+
+  if (*password)
+    return TRUE;
+
   if (username_node)
     g_object_unref (username_node);
   if (password_node)
@@ -538,6 +682,47 @@ ephy_web_dom_utils_find_form_auth_elements (WebKitDOMHTMLFormElement *form,
   return FALSE;
 }
 
+/* ephy_web_dom_utils_find_form_auth_elements() is great, but it only returns
+ * something useful if it can pair a username node with a password node. This
+ * function says whether the form has a password node or not, even if Epiphany
+ * isn't smart enough to match it to a username node.
+ *
+ * Ideally we'd also detect credit card data or other sensitive stuff, but
+ * there's no standard way to do so, so don't try.
+ */
+gboolean
+ephy_web_dom_utils_form_contains_sensitive_element (WebKitDOMHTMLFormElement *form)
+{
+  WebKitDOMHTMLCollection *elements;
+  guint i, n_elements;
+  gboolean found_auth_element = FALSE;
+
+  elements = webkit_dom_html_form_element_get_elements (form);
+  n_elements = webkit_dom_html_collection_get_length (elements);
+
+  for (i = 0; i < n_elements && !found_auth_element; i++) {
+    WebKitDOMNode *element;
+    char *element_type;
+
+    element = webkit_dom_html_collection_item (elements, i);
+    if (!WEBKIT_DOM_IS_HTML_INPUT_ELEMENT (element))
+      continue;
+
+    g_object_get (element, "type", &element_type, NULL);
+
+    if (g_strcmp0 (element_type, "password") == 0 ||
+        g_strcmp0 (element_type, "adminpw") == 0) {
+      found_auth_element = TRUE;
+    }
+
+    g_free (element_type);
+  }
+
+  g_object_unref (elements);
+
+  return found_auth_element;
+}
+
 /**
  * ephy_web_dom_utils_get_absolute_position_for_element:
  * @element: the #WebKitDOMElement.
