@@ -2,18 +2,20 @@
 /*
  *  Copyright © 2002 Marco Pesenti Gritti
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2, or (at your option)
- *  any later version.
+ *  This file is part of Epiphany.
  *
- *  This program is distributed in the hope that it will be useful,
+ *  Epiphany is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ *
+ *  Epiphany is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ *  along with Epiphany.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -240,3 +242,90 @@ ephy_string_commandline_args_to_uris (char **arguments, GError **error)
 
   return args;
 }
+
+char *
+ephy_string_find_and_replace (const char *haystack,
+                              const char *to_find,
+                              const char *to_repl)
+{
+  GString *string;
+  const char *needle;
+  gsize to_find_len;
+  gsize to_repl_len;
+  gsize pos = 0;
+  gsize i = 0;
+
+  string = g_string_new (haystack);
+  to_find_len = strlen (to_find);
+  to_repl_len = strlen (to_repl);
+
+  while ((needle = strstr (haystack, to_find)) != NULL) {
+    pos += needle - haystack;
+    g_string_erase (string, pos + i * (to_repl_len - to_find_len), to_find_len);
+    g_string_insert (string, pos + i * (to_repl_len - to_find_len), to_repl);
+    haystack = needle + to_find_len;
+    pos += to_find_len;
+    i++;
+  }
+
+  return g_string_free (string, FALSE);
+}
+
+char **
+ephy_strv_append (const char * const *strv,
+                  const char         *str)
+{
+  char **new_strv;
+  char **n;
+  const char * const *s;
+  guint len;
+
+  if (g_strv_contains (strv, str))
+    return g_strdupv ((char **)strv);
+
+  /* Needs room for one more string than before, plus one for trailing NULL. */
+  len = g_strv_length ((char **)strv);
+  new_strv = g_malloc ((len + 1 + 1) * sizeof (char *));
+  n = new_strv;
+  s = strv;
+
+  while (*s != NULL) {
+    *n = g_strdup (*s);
+    n++;
+    s++;
+  }
+  new_strv[len] = g_strdup (str);
+  new_strv[len + 1] = NULL;
+
+  return new_strv;
+}
+
+char **
+ephy_strv_remove (const char * const *strv,
+                  const char         *str)
+{
+  char **new_strv;
+  char **n;
+  const char * const *s;
+  guint len;
+
+  if (!g_strv_contains (strv, str))
+    return g_strdupv ((char **)strv);
+
+  /* Needs room for one fewer string than before, plus one for trailing NULL. */
+  len = g_strv_length ((char **)strv);
+  new_strv = g_malloc ((len - 1 + 1) * sizeof (char *));
+  n = new_strv;
+  s = strv;
+
+  while (*s != NULL) {
+    if (strcmp (*s, str) != 0) {
+      *n = g_strdup (*s);
+      n++;
+    }
+    s++;
+  }
+  new_strv[len - 1] = NULL;
+
+  return new_strv;
+}
