@@ -280,10 +280,12 @@ notebook_drag_data_received_cb (GtkWidget        *widget,
   g_signal_stop_emission_by_name (widget, "drag-data-received");
 
   if (g_settings_get_boolean (EPHY_SETTINGS_LOCKDOWN,
-                              EPHY_PREFS_LOCKDOWN_ARBITRARY_URL)) return;
+                              EPHY_PREFS_LOCKDOWN_ARBITRARY_URL))
+    return;
 
   data = gtk_selection_data_get_data (selection_data);
-  if (gtk_selection_data_get_length (selection_data) <= 0 || data == NULL) return;
+  if (gtk_selection_data_get_length (selection_data) <= 0 || data == NULL)
+    return;
 
   window = EPHY_WINDOW (gtk_widget_get_toplevel (widget));
   notebook = ephy_window_get_notebook (window);
@@ -303,9 +305,10 @@ notebook_drag_data_received_cb (GtkWidget        *widget,
     int i;
 
     uris = gtk_selection_data_get_uris (selection_data);
-    if (uris == NULL) return;
+    if (uris == NULL)
+      return;
 
-    for (i = 0; uris[i] != NULL && i < INSANE_NUMBER_OF_URLS; i++) {
+    for (i = 0; i < INSANE_NUMBER_OF_URLS && uris[i] != NULL; i++) {
       embed = ephy_link_open (EPHY_LINK (notebook), uris[i], embed,
                               (embed && i == 0) ? 0 : EPHY_LINK_NEW_TAB);
     }
@@ -465,7 +468,6 @@ ephy_notebook_constructed (GObject *object)
   EphyNotebook *notebook = EPHY_NOTEBOOK (object);
   GtkWidget *hbox;
   GtkWidget *button;
-  GtkWidget *popover;
 
   G_OBJECT_CLASS (ephy_notebook_parent_class)->constructed (object);
 
@@ -489,9 +491,9 @@ ephy_notebook_constructed (GObject *object)
   gtk_widget_show (button);
 
   notebook->tab_menu = g_menu_new ();
-  popover = gtk_popover_new (button);
-  gtk_popover_bind_model (GTK_POPOVER (popover), G_MENU_MODEL (notebook->tab_menu), "win");
-  gtk_menu_button_set_popover (GTK_MENU_BUTTON (button), popover);
+  /* Remove this when popover menus become scrollable. */
+  gtk_menu_button_set_use_popover (GTK_MENU_BUTTON (button), FALSE);
+  gtk_menu_button_set_menu_model (GTK_MENU_BUTTON (button), G_MENU_MODEL (notebook->tab_menu));
 }
 
 static void
@@ -574,7 +576,7 @@ ephy_notebook_rebuild_tab_menu (EphyNotebook *notebook)
     text = get_nth_tab_label_text (GTK_NOTEBOOK (notebook), i);
     ellipsized_text = ellipsize_tab_label (text);
     item = g_menu_item_new (ellipsized_text, NULL);
-    g_menu_item_set_action_and_target (item, "show-tab", "u", (guint)i, NULL);
+    g_menu_item_set_action_and_target (item, "win.show-tab", "u", (guint)i, NULL);
     g_menu_append_item (notebook->tab_menu, item);
     g_free (ellipsized_text);
     g_object_unref (item);
