@@ -1,32 +1,33 @@
-/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
+/* -*- Mode: C; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
 /*
  *  Copyright © 2000-2003 Marco Pesenti Gritti
+ *  Copyright © 2017 Igalia S.L.
  *
- *  This program is free software; you can redistribute it and/or modify
+ *  This file is part of Epiphany.
+ *
+ *  Epiphany is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2, or (at your option)
- *  any later version.
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
+ *  Epiphany is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ *  along with Epiphany.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
 #include "popup-commands.h"
 
-#include "ephy-bookmarks-ui.h"
 #include "ephy-downloads-manager.h"
 #include "ephy-embed-container.h"
 #include "ephy-embed-utils.h"
 #include "ephy-file-chooser.h"
 #include "ephy-file-helpers.h"
 #include "ephy-prefs.h"
-#include "ephy-private.h"
 #include "ephy-settings.h"
 #include "ephy-shell.h"
 #include "ephy-web-view.h"
@@ -85,63 +86,39 @@ view_in_destination (EphyWindow     *window,
 }
 
 void
-popup_cmd_link_in_new_window (GtkAction  *action,
-                              EphyWindow *window)
+popup_cmd_link_in_new_window (GSimpleAction *action,
+                              GVariant      *parameter,
+                              gpointer       user_data)
 {
-  view_in_destination (window, "link-uri", NEW_WINDOW);
+  view_in_destination (EPHY_WINDOW (user_data), "link-uri", NEW_WINDOW);
 }
 
 void
-popup_cmd_media_in_new_window (GtkAction  *action,
-                               EphyWindow *window)
+popup_cmd_media_in_new_window (GSimpleAction *action,
+                               GVariant      *parameter,
+                               gpointer       user_data)
 {
-  view_in_destination (window, "media-uri", NEW_WINDOW);
-}
-
-void
-popup_cmd_bookmark_link (GtkAction  *action,
-                         EphyWindow *window)
-{
-  EphyEmbedEvent *event;
-  WebKitHitTestResult *result;
-  const char *title;
-  const char *location;
-
-  event = ephy_window_get_context_event (window);
-
-  result = ephy_embed_event_get_hit_test_result (event);
-  if (!webkit_hit_test_result_context_is_link (result)) {
-    return;
-  }
-
-  location = webkit_hit_test_result_get_link_uri (result);
-  title = webkit_hit_test_result_get_link_title (result);
-  if (!title) {
-    title = webkit_hit_test_result_get_link_label (result);
-  }
-
-  ephy_bookmarks_ui_add_bookmark (GTK_WINDOW (window), location, title);
+  view_in_destination (EPHY_WINDOW (user_data), "media-uri", NEW_WINDOW);
 }
 
 static void
 popup_cmd_copy_to_clipboard (EphyWindow *window, const char *text)
 {
-  gtk_clipboard_set_text (gtk_clipboard_get (GDK_NONE),
-                          text, -1);
-  gtk_clipboard_set_text (gtk_clipboard_get (GDK_SELECTION_PRIMARY),
+  gtk_clipboard_set_text (gtk_clipboard_get_default (gdk_display_get_default ()),
                           text, -1);
 }
 
 void
-popup_cmd_copy_link_address (GtkAction  *action,
-                             EphyWindow *window)
+popup_cmd_copy_link_address (GSimpleAction *action,
+                             GVariant      *parameter,
+                             gpointer       user_data)
 {
   EphyEmbedEvent *event;
   guint context;
   const char *address;
   GValue value = { 0, };
 
-  event = ephy_window_get_context_event (window);
+  event = ephy_window_get_context_event (EPHY_WINDOW (user_data));
   g_return_if_fail (event != NULL);
 
   context = ephy_embed_event_get_context (event);
@@ -153,7 +130,7 @@ popup_cmd_copy_link_address (GtkAction  *action,
     if (g_str_has_prefix (address, "mailto:"))
       address = address + 7;
 
-    popup_cmd_copy_to_clipboard (window, address);
+    popup_cmd_copy_to_clipboard (EPHY_WINDOW (user_data), address);
     g_value_unset (&value);
   }
 }
@@ -176,25 +153,24 @@ filename_suggested_cb (EphyDownload        *download,
                        const char          *suggested_filename,
                        SavePropertyURLData *data)
 {
-  EphyFileChooser *dialog;
+  GtkFileChooser *dialog;
   char *sanitized_filename;
 
-  dialog = ephy_file_chooser_new (data->title,
-                                  GTK_WIDGET (data->window),
-                                  GTK_FILE_CHOOSER_ACTION_SAVE,
-                                  EPHY_FILE_FILTER_NONE);
-  gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);
-  gtk_file_chooser_set_do_overwrite_confirmation (GTK_FILE_CHOOSER (dialog), TRUE);
+  dialog = ephy_create_file_chooser (data->title,
+                                     GTK_WIDGET (data->window),
+                                     GTK_FILE_CHOOSER_ACTION_SAVE,
+                                     EPHY_FILE_FILTER_NONE);
+  gtk_file_chooser_set_do_overwrite_confirmation (dialog, TRUE);
 
   sanitized_filename = ephy_sanitize_filename (g_strdup (suggested_filename));
-  gtk_file_chooser_set_current_name (GTK_FILE_CHOOSER (dialog), sanitized_filename);
+  gtk_file_chooser_set_current_name (dialog, sanitized_filename);
   g_free (sanitized_filename);
 
-  if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT) {
+  if (gtk_native_dialog_run (GTK_NATIVE_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT) {
     char *uri;
     WebKitDownload *webkit_download;
 
-    uri = gtk_file_chooser_get_uri (GTK_FILE_CHOOSER (dialog));
+    uri = gtk_file_chooser_get_uri (dialog);
     ephy_download_set_destination_uri (download, uri);
     g_free (uri);
 
@@ -210,7 +186,7 @@ filename_suggested_cb (EphyDownload        *download,
                      g_object_unref);
   }
 
-  gtk_widget_destroy (GTK_WIDGET (dialog));
+  g_object_unref (dialog);
   g_free (data->title);
   g_object_unref (data->window);
   g_free (data);
@@ -227,7 +203,7 @@ save_property_url (const char *title,
   const char *location;
   EphyDownload *download;
   SavePropertyURLData *data;
-  GValue value = { 0, };
+  GValue value = G_VALUE_INIT;
 
   event = ephy_window_get_context_event (window);
   g_return_if_fail (event != NULL);
@@ -246,24 +222,27 @@ save_property_url (const char *title,
 }
 
 void
-popup_cmd_download_link_as (GtkAction  *action,
-                            EphyWindow *window)
+popup_cmd_download_link_as (GSimpleAction *action,
+                            GVariant      *parameter,
+                            gpointer       user_data)
 {
-  save_property_url (_("Save Link As"), window, "link-uri");
+  save_property_url (_("Save Link As"), EPHY_WINDOW (user_data), "link-uri");
 }
 
 void
-popup_cmd_save_image_as (GtkAction  *action,
-                         EphyWindow *window)
+popup_cmd_save_image_as (GSimpleAction *action,
+                         GVariant      *parameter,
+                         gpointer       user_data)
 {
-  save_property_url (_("Save Image As"), window, "image-uri");
+  save_property_url (_("Save Image As"), EPHY_WINDOW (user_data), "image-uri");
 }
 
 void
-popup_cmd_save_media_as (GtkAction  *action,
-                         EphyWindow *window)
+popup_cmd_save_media_as (GSimpleAction *action,
+                         GVariant      *parameter,
+                         gpointer       user_data)
 {
-  save_property_url (_("Save Media As"), window, "media-uri");
+  save_property_url (_("Save Media As"), EPHY_WINDOW (user_data), "media-uri");
 }
 
 static void
@@ -279,8 +258,9 @@ background_download_completed (EphyDownload *download,
 }
 
 void
-popup_cmd_set_image_as_background (GtkAction  *action,
-                                   EphyWindow *window)
+popup_cmd_set_image_as_background (GSimpleAction *action,
+                                   GVariant      *parameter,
+                                   gpointer       user_data)
 {
   EphyEmbedEvent *event;
   const char *location;
@@ -288,7 +268,7 @@ popup_cmd_set_image_as_background (GtkAction  *action,
   GValue value = { 0, };
   EphyDownload *download;
 
-  event = ephy_window_get_context_event (window);
+  event = ephy_window_get_context_event (EPHY_WINDOW (user_data));
   g_return_if_fail (event != NULL);
 
   ephy_embed_event_get_property (event, "image-uri", &value);
@@ -307,7 +287,7 @@ popup_cmd_set_image_as_background (GtkAction  *action,
   g_object_unref (download);
 
   g_signal_connect (download, "completed",
-                    G_CALLBACK (background_download_completed), window);
+                    G_CALLBACK (background_download_completed), user_data);
 
   g_value_unset (&value);
   g_free (base);
@@ -332,48 +312,54 @@ popup_cmd_copy_location (EphyWindow *window,
 }
 
 void
-popup_cmd_copy_image_location (GtkAction  *action,
-                               EphyWindow *window)
+popup_cmd_copy_image_location (GSimpleAction *action,
+                               GVariant      *parameter,
+                               gpointer       user_data)
 {
-  popup_cmd_copy_location (window, "image-uri");
+  popup_cmd_copy_location (EPHY_WINDOW (user_data), "image-uri");
 }
 
 void
-popup_cmd_copy_media_location (GtkAction  *action,
-                               EphyWindow *window)
+popup_cmd_copy_media_location (GSimpleAction *action,
+                               GVariant      *parameter,
+                               gpointer       user_data)
 {
-  popup_cmd_copy_location (window, "media-uri");
+  popup_cmd_copy_location (EPHY_WINDOW (user_data), "media-uri");
 }
 
 void
-popup_cmd_link_in_new_tab (GtkAction  *action,
-                           EphyWindow *window)
+popup_cmd_link_in_new_tab (GSimpleAction *action,
+                           GVariant      *parameter,
+                           gpointer       user_data)
 {
-  view_in_destination (window, "link-uri", NEW_TAB);
+  view_in_destination (EPHY_WINDOW (user_data), "link-uri", NEW_TAB);
 }
 
 void
-popup_cmd_view_image_in_new_tab (GtkAction  *action,
-                                 EphyWindow *window)
+popup_cmd_view_image_in_new_tab (GSimpleAction *action,
+                                 GVariant      *parameter,
+                                 gpointer       user_data)
 {
-  view_in_destination (window, "image-uri", NEW_TAB);
+  view_in_destination (EPHY_WINDOW (user_data), "image-uri", NEW_TAB);
 }
 
 void
-popup_cmd_media_in_new_tab (GtkAction  *action,
-                            EphyWindow *window)
+popup_cmd_media_in_new_tab (GSimpleAction *action,
+                            GVariant      *parameter,
+                            gpointer       user_data)
 {
-  view_in_destination (window, "media-uri", NEW_TAB);
+  view_in_destination (EPHY_WINDOW (user_data), "media-uri", NEW_TAB);
 }
 
 void
-popup_cmd_link_in_incognito_window (GtkAction  *action,
-                                    EphyWindow *window)
+popup_cmd_link_in_incognito_window (GSimpleAction *action,
+                                    GVariant      *parameter,
+                                    gpointer       user_data)
 {
   EphyEmbedEvent *event;
   GValue value = { 0, };
 
-  event = ephy_window_get_context_event (window);
+  event = ephy_window_get_context_event (EPHY_WINDOW (user_data));
   g_assert (event != NULL);
 
   ephy_embed_event_get_property (event, "link-uri", &value);
@@ -382,21 +368,21 @@ popup_cmd_link_in_incognito_window (GtkAction  *action,
 }
 
 void
-popup_cmd_search_selection (GtkAction  *action,
-                            EphyWindow *window)
+popup_cmd_search_selection (GSimpleAction *action,
+                            GVariant      *parameter,
+                            gpointer       user_data)
 {
   EphyEmbed *embed, *new_embed;
-  const char *text;
+  const char *search_term;
   char *search_url;
 
-  embed = ephy_embed_container_get_active_child
-            (EPHY_EMBED_CONTAINER (window));
+  embed = ephy_embed_container_get_active_child (EPHY_EMBED_CONTAINER (user_data));
   g_assert (EPHY_IS_EMBED (embed));
 
-  text = g_object_get_data (G_OBJECT (action), "selection");
-  search_url = ephy_embed_utils_autosearch_address (text);
+  search_term = g_variant_get_string (parameter, NULL);
+  search_url = ephy_embed_utils_autosearch_address (search_term);
   new_embed = ephy_shell_new_tab (ephy_shell_get_default (),
-                                  window, embed, EPHY_NEW_TAB_APPEND_AFTER);
+                                  EPHY_WINDOW (user_data), embed, EPHY_NEW_TAB_APPEND_AFTER);
   ephy_web_view_load_url (ephy_embed_get_web_view (new_embed), search_url);
   g_free (search_url);
 }
