@@ -34,13 +34,14 @@
 #include "ephy-bookmarks-manager.h"
 #include "ephy-debug.h"
 #include "ephy-file-helpers.h"
-#include "ephy-form-auth-data.h"
 #include "ephy-history-service.h"
+#include "ephy-password-manager.h"
 #include "ephy-prefs.h"
 #include "ephy-profile-utils.h"
 #include "ephy-search-engine-manager.h"
 #include "ephy-settings.h"
 #include "ephy-sqlite-connection.h"
+#include "ephy-sync-utils.h"
 #include "ephy-uri-tester-shared.h"
 #include "ephy-web-app-utils.h"
 
@@ -223,11 +224,14 @@ migrate_new_urls_table (void)
   GError *error = NULL;
 
   filename = g_build_filename (ephy_dot_dir (), EPHY_HISTORY_FILE, NULL);
+  if (!g_file_test (filename, G_FILE_TEST_EXISTS))
+    return;
+
   history_database = ephy_sqlite_connection_new (EPHY_SQLITE_CONNECTION_MODE_READWRITE);
   ephy_sqlite_connection_open (history_database, filename, &error);
 
   if (error) {
-    g_warning ("Failed to open history database: %s\n", error->message);
+    g_warning ("Failed to open history database: %s", error->message);
     g_error_free (error);
     g_free (filename);
     return;
@@ -238,7 +242,7 @@ migrate_new_urls_table (void)
                                   "ADD COLUMN thumbnail_update_time INTEGER DEFAULT 0",
                                   &error);
   if (error) {
-    g_warning ("Failed to add new column to table in history backend: %s\n",
+    g_warning ("Failed to add new column to table in history backend: %s",
                error->message);
     g_error_free (error);
     error = NULL;
@@ -248,7 +252,7 @@ migrate_new_urls_table (void)
                                   "ADD COLUMN hidden_from_overview INTEGER DEFAULT 0",
                                   &error);
   if (error) {
-    g_warning ("Failed to add new column to table in history backend: %s\n",
+    g_warning ("Failed to add new column to table in history backend: %s",
                error->message);
     g_error_free (error);
     error = NULL;
@@ -281,7 +285,7 @@ store_form_auth_data_cb (GObject      *object,
 {
   GError *error = NULL;
 
-  if (ephy_form_auth_data_store_finish (res, &error) == FALSE) {
+  if (ephy_password_manager_store_finish (res, &error) == FALSE) {
     g_warning ("Couldn't store a form password: %s", error->message);
     g_error_free (error);
     goto out;
@@ -308,8 +312,8 @@ load_collection_items_cb (SecretCollection *collection,
   SecretValue *secret;
   GList *l;
   GHashTable *attributes, *t;
-  const char *server, *username, *form_username, *form_password, *password;
-  char *actual_server;
+  const char *server, *username, *username_field, *password_field, *password;
+  char *origin;
   SoupURI *uri;
   GError *error = NULL;
   GList *items;
@@ -337,21 +341,21 @@ load_collection_items_cb (SecretCollection *collection,
       username = g_hash_table_lookup (attributes, "user");
       uri = soup_uri_new (server);
       t = soup_form_decode (uri->query);
-      form_username = g_hash_table_lookup (t, FORM_USERNAME_KEY);
-      form_password = g_hash_table_lookup (t, FORM_PASSWORD_KEY);
+      username_field = g_hash_table_lookup (t, USERNAME_FIELD_KEY);
+      password_field = g_hash_table_lookup (t, PASSWORD_FIELD_KEY);
       soup_uri_set_query (uri, NULL);
-      actual_server = soup_uri_to_string (uri, FALSE);
+      origin = soup_uri_to_string (uri, FALSE);
       secret_item_load_secret_sync (item, NULL, NULL);
       secret = secret_item_get_secret (item);
       password = secret_value_get (secret, NULL);
-      ephy_form_auth_data_store (actual_server,
-                                 form_username,
-                                 form_password,
-                                 username,
-                                 password,
-                                 (GAsyncReadyCallback)store_form_auth_data_cb,
-                                 g_hash_table_ref (attributes));
-      g_free (actual_server);
+      ephy_password_manager_store_raw (origin,
+                                       username,
+                                       password,
+                                       username_field,
+                                       password_field,
+                                       (GAsyncReadyCallback)store_form_auth_data_cb,
+                                       g_hash_table_ref (attributes));
+      g_free (origin);
       secret_value_unref (secret);
       g_hash_table_unref (t);
       soup_uri_free (uri);
@@ -439,7 +443,7 @@ migrate_insecure_password (SecretItem *item)
   const char *original_uri;
 
   attributes = secret_item_get_attributes (item);
-  original_uri = g_hash_table_lookup (attributes, URI_KEY);
+  original_uri = g_hash_table_lookup (attributes, ORIGIN_KEY);
   original_origin = webkit_security_origin_new_for_uri (original_uri);
   if (original_origin == NULL) {
     g_warning ("Failed to convert URI %s to a security origin, insecure password will not be migrated", original_uri);
@@ -458,7 +462,7 @@ migrate_insecure_password (SecretItem *item)
     new_uri = webkit_security_origin_to_string (new_origin);
     webkit_security_origin_unref (new_origin);
 
-    g_hash_table_replace (attributes, g_strdup (URI_KEY), new_uri);
+    g_hash_table_replace (attributes, g_strdup (ORIGIN_KEY), new_uri);
     secret_item_set_attributes_sync (item, EPHY_FORM_PASSWORD_SCHEMA, attributes, NULL, &error);
     if (error != NULL) {
       g_warning ("Failed to convert URI %s to https://, insecure password will not be migrated: %s", original_uri, error->message);
@@ -573,7 +577,7 @@ parse_rdf_lang_tag (xmlNode  *child,
 
   locales = g_get_language_names ();
 
-  for (i = 0; locales[i] && i < *best_match; i++) {
+  for (i = 0; i < *best_match && locales[i]; i++) {
     if (!strcmp (locales[i], this_language)) {
       /* if we've already encountered a less accurate
        * translation, then free it */
@@ -587,8 +591,10 @@ parse_rdf_lang_tag (xmlNode  *child,
     }
   }
 
-  if (lang) xmlFree (lang);
-  if (*value != content) xmlFree (content);
+  if (lang)
+    xmlFree (lang);
+  if (*value != content)
+    xmlFree (content);
 }
 
 static void
@@ -641,11 +647,15 @@ parse_rdf_item (EphyBookmarksManager *manager,
 
   if (link) {
     EphyBookmark *bookmark;
+    char *id;
 
     g_sequence_sort (tags, (GCompareDataFunc)ephy_bookmark_tags_compare, NULL);
-    bookmark = ephy_bookmark_new ((const char *)link, (const char *)title, tags);
+    id = ephy_sync_utils_get_random_sync_id ();
+    bookmark = ephy_bookmark_new ((const char *)link, (const char *)title, tags, id);
     ephy_bookmarks_manager_add_bookmark (manager, bookmark);
+
     g_object_unref (bookmark);
+    g_free (id);
   } else {
     g_sequence_free (tags);
   }
@@ -678,7 +688,7 @@ migrate_bookmarks (void)
 
   doc = xmlParseFile (filename);
   if (doc == NULL) {
-    g_warning ("Failed to re-import the bookmarks. All bookmarks lost!\n");
+    g_warning ("Failed to re-import the bookmarks. All bookmarks lost!");
     goto out;
   }
 
@@ -969,6 +979,224 @@ migrate_search_engines (void)
   g_object_unref (search_engine_manager);
 }
 
+static void
+migrate_icon_database (void)
+{
+  char *path;
+  GError *error = NULL;
+
+  /* Favicons used to be saved here by mistake. Delete them! */
+  path = g_build_filename (g_get_user_cache_dir (), "icondatabase", NULL);
+  ephy_file_delete_dir_recursively (path, &error);
+
+  if (error) {
+    if (!g_error_matches (error, G_FILE_ERROR, G_FILE_ERROR_NOENT))
+      g_warning ("Failed to delete %s: %s", path, error->message);
+    g_error_free (error);
+  }
+
+  g_free (path);
+}
+
+static void
+migrate_passwords_to_firefox_sync_passwords (void)
+{
+  GHashTable *attributes;
+  GList *passwords = NULL;
+  GError *error = NULL;
+  int default_profile_migration_version;
+
+  /* Similar to the insecure passwords migration, we want to migrate passwords
+   * to Firefox Sync passwords only once since saved passwords are stored
+   * globally and not per profile. This won't affect password lookup for web
+   * apps because this migration only adds a couple of new fields to the
+   * password schema, fields that are not taken into consideration when
+   * querying passwords.
+   */
+  default_profile_migration_version = ephy_profile_utils_get_migration_version_for_profile_dir (ephy_default_dot_dir ());
+  if (default_profile_migration_version >= EPHY_FIREFOX_SYNC_PASSWORDS_MIGRATION_VERSION)
+    return;
+
+  attributes = secret_attributes_build (EPHY_FORM_PASSWORD_SCHEMA, NULL);
+  passwords = secret_service_search_sync (NULL, EPHY_FORM_PASSWORD_SCHEMA, attributes,
+                                          SECRET_SEARCH_ALL | SECRET_SEARCH_UNLOCK | SECRET_SEARCH_LOAD_SECRETS,
+                                          NULL, &error);
+  if (error) {
+    g_warning ("Failed to search the password schema: %s", error->message);
+    goto out;
+  }
+
+  secret_service_clear_sync (NULL, EPHY_FORM_PASSWORD_SCHEMA,
+                             attributes, NULL, &error);
+  if (error) {
+    g_warning ("Failed to clear the password schema: %s", error->message);
+    goto out;
+  }
+
+  for (GList *l = passwords; l && l->data; l = l->next) {
+    SecretItem *item = (SecretItem *)l->data;
+    SecretValue *value = secret_item_get_secret (item);
+    GHashTable *attrs = secret_item_get_attributes (item);
+    const char *origin = g_hash_table_lookup (attrs, ORIGIN_KEY);
+    const char *username = g_hash_table_lookup (attrs, USERNAME_KEY);
+    char *uuid = g_uuid_string_random ();
+    char *label;
+
+    g_hash_table_insert (attrs, g_strdup (ID_KEY), g_strdup_printf ("{%s}", uuid));
+    g_hash_table_insert (attrs, g_strdup (SERVER_TIME_MODIFIED_KEY), g_strdup ("0"));
+
+    if (username)
+      label = g_strdup_printf ("Password for %s in a form in %s", username, origin);
+    else
+      label = g_strdup_printf ("Password in a form in %s", origin);
+    secret_service_store_sync (NULL, EPHY_FORM_PASSWORD_SCHEMA,
+                               attrs, NULL, label,
+                               value, NULL, &error);
+    if (error) {
+      g_warning ("Failed to store password: %s", error->message);
+      g_clear_pointer (&error, g_error_free);
+    }
+
+    g_free (label);
+    g_free (uuid);
+    secret_value_unref (value);
+    g_hash_table_unref (attrs);
+  }
+
+out:
+  if (error)
+    g_error_free (error);
+  g_hash_table_unref (attributes);
+  g_list_free_full (passwords, g_object_unref);
+}
+
+static void
+migrate_history_to_firefox_sync_history (void)
+{
+  EphySQLiteConnection *history_db = NULL;
+  EphySQLiteStatement *statement = NULL;
+  GError *error = NULL;
+  char *history_filename;
+  const char *sql_query;
+
+  history_filename = g_build_filename (ephy_dot_dir (), EPHY_HISTORY_FILE, NULL);
+  if (!g_file_test (history_filename, G_FILE_TEST_EXISTS)) {
+    LOG ("There is no history to migrate...");
+    goto out;
+  }
+
+  history_db = ephy_sqlite_connection_new (EPHY_SQLITE_CONNECTION_MODE_READWRITE);
+  ephy_sqlite_connection_open (history_db, history_filename, &error);
+  if (error) {
+    g_warning ("Failed to open history database: %s", error->message);
+    goto out;
+  }
+
+  /* Add new sync_id column. All sync ids will default to NULL. */
+  sql_query = "ALTER TABLE urls ADD COLUMN sync_id LONGVARCAR";
+  ephy_sqlite_connection_execute (history_db, sql_query, &error);
+  if (error) {
+    /* SQLite gives only a generic error code if the column already exists, so
+     * assume this migrator has been run already and don't print a warning. */
+    goto out;
+  }
+
+  /* Update visit timestamps to microseconds. */
+  sql_query = "UPDATE urls SET last_visit_time = last_visit_time * 1000000";
+  ephy_sqlite_connection_execute (history_db, sql_query, &error);
+  if (error) {
+    g_warning ("Failed to update last visit time to microseconds: %s", error->message);
+    goto out;
+  }
+
+  sql_query = "UPDATE visits SET visit_time = visit_time * 1000000";
+  ephy_sqlite_connection_execute (history_db, sql_query, &error);
+  if (error)
+    g_warning ("Failed to update visit time to microseconds: %s", error->message);
+
+out:
+  g_free (history_filename);
+  if (history_db)
+    g_object_unref (history_db);
+  if (statement)
+    g_object_unref (statement);
+  if (error)
+    g_error_free (error);
+}
+
+static void
+migrate_passwords_add_target_origin (void)
+{
+  GHashTable *attributes;
+  GList *passwords = NULL;
+  GError *error = NULL;
+  int default_profile_migration_version;
+
+  /* Similar to Firefox Sync and the insecure passwords migrations.
+   * This is also a migration that runs once, and not for each profile
+   * Adds target_origin field to all existing records,
+   * with the same value as origin.
+   */
+  default_profile_migration_version = ephy_profile_utils_get_migration_version_for_profile_dir (ephy_default_dot_dir ());
+  if (default_profile_migration_version >= EPHY_TARGET_ORIGIN_MIGRATION_VERSION)
+    return;
+
+  attributes = secret_attributes_build (EPHY_FORM_PASSWORD_SCHEMA, NULL);
+  passwords = secret_service_search_sync (NULL, EPHY_FORM_PASSWORD_SCHEMA, attributes,
+                                          SECRET_SEARCH_ALL | SECRET_SEARCH_UNLOCK | SECRET_SEARCH_LOAD_SECRETS,
+                                          NULL, &error);
+  if (error) {
+    g_warning ("Failed to search the password schema: %s", error->message);
+    goto out;
+  }
+
+  secret_service_clear_sync (NULL, EPHY_FORM_PASSWORD_SCHEMA,
+                             attributes, NULL, &error);
+  if (error) {
+    g_warning ("Failed to clear the password schema: %s", error->message);
+    goto out;
+  }
+
+  for (GList *l = passwords; l && l->data; l = l->next) {
+    SecretItem *item = (SecretItem *)l->data;
+    SecretValue *value = secret_item_get_secret (item);
+    GHashTable *attrs = secret_item_get_attributes (item);
+    const char *origin = g_hash_table_lookup (attrs, ORIGIN_KEY);
+    const char *username = g_hash_table_lookup (attrs, USERNAME_KEY);
+    const char *target_origin = g_hash_table_lookup (attrs, TARGET_ORIGIN_KEY);
+    char *label;
+
+    /* In most cases target_origin has the same value as origin
+     * We don't have a way of figuring out the correct value retroactively,
+     * so just use the origin value.
+    */
+    if (target_origin == NULL)
+      g_hash_table_insert (attrs, g_strdup (TARGET_ORIGIN_KEY), g_strdup (origin));
+
+    if (username)
+      label = g_strdup_printf ("Password for %s in a form in %s", username, origin);
+    else
+      label = g_strdup_printf ("Password in a form in %s", origin);
+    secret_service_store_sync (NULL, EPHY_FORM_PASSWORD_SCHEMA,
+                               attrs, NULL, label,
+                               value, NULL, &error);
+    if (error) {
+      g_warning ("Failed to store password: %s", error->message);
+      g_clear_pointer (&error, g_error_free);
+    }
+
+    g_free (label);
+    secret_value_unref (value);
+    g_hash_table_unref (attrs);
+  }
+
+out:
+  if (error)
+    g_error_free (error);
+  g_hash_table_unref (attributes);
+  g_list_free_full (passwords, g_object_unref);
+}
+
 static void
 migrate_nothing (void)
 {
@@ -1000,6 +1228,10 @@ const EphyProfileMigrator migrators[] = {
   /* 15 */ migrate_permissions,
   /* 16 */ migrate_settings,
   /* 17 */ migrate_search_engines,
+  /* 18 */ migrate_icon_database,
+  /* 19 */ migrate_passwords_to_firefox_sync_passwords,
+  /* 20 */ migrate_history_to_firefox_sync_history,
+  /* 21 */ migrate_passwords_add_target_origin,
 };
 
 static gboolean
@@ -1008,8 +1240,6 @@ ephy_migrator (void)
   int latest, i;
   EphyProfileMigrator m;
 
-  g_assert (EPHY_PROFILE_MIGRATION_VERSION == G_N_ELEMENTS (migrators));
-
   /* Always try to migrate the data from the old profile dir at the
    * very beginning. */
   migrate_profile_gnome2_to_xdg ();
@@ -1080,6 +1310,8 @@ main (int argc, char *argv[])
   bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
   textdomain (GETTEXT_PACKAGE);
 
+  g_assert (EPHY_PROFILE_MIGRATION_VERSION == G_N_ELEMENTS (migrators));
+
   option_group = g_option_group_new ("ephy-profile-migrator",
                                      N_("Web profile migrator"),
                                      N_("Web profile migrator options"),
